//
//  Copyright (C) 2011 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gtk;
using Gdk;
using Granite;
using SQLHeavy;
using Gee;

namespace Footnote.Widgets {
	/**
	 * A widget that offers an easy visual way for managing tags
	 */
	public class TaggingArea : HBox {
		
		public class Tag : Box {

			private Label _tag;
			public string tag {
				get { return _tag.label; }
				set { _tag.label = value; }
			}
			private TaggingArea tagparent;
			
			public override bool draw (Cairo.Context ctx){
				int w = this.get_allocated_width ();
				int h = this.get_allocated_height ();
				Granite.Drawing.Utilities.cairo_rounded_rectangle(ctx, 4, 4, w-5, h-5, 2);
				ctx.set_source_rgba (1.0, 1.0, 1.0, 1.0);
				ctx.fill_preserve ();
				ctx.set_source_rgba (0.2, 0.2, 0.2, 1.0);
				ctx.set_line_width (0.5);
				ctx.stroke ();
				ctx.move_to (24,4);
				ctx.line_to (24,h-1);
				ctx.stroke ();
				return base.draw (ctx);
			}
			
			/**
			 * create a new tag
			 * @param name The displayed name of the tag
			 */
			public Tag (string name, TaggingArea parent){
				this.tagparent = parent;
				
				var box = new HBox (false, 0);
				box.margin = 3;
				_tag = new Label(name);
				_tag.margin = 3;
				_tag.margin_right = 6;
				_tag.margin_left = 6;

				Color white;
				Color.parse ("#ffffff", out white);
				_tag.modify_bg (StateType.NORMAL, white);

				Color grey;
				Color.parse ("#5D5D5D", out grey);
				_tag.modify_fg (StateType.NORMAL, grey);
				box.pack_end (_tag, false, false, 0);

				var close = new EventBox ();
				close.visible_window = false;
				close.add (new Image.from_stock (Gtk.Stock.CLOSE, IconSize.MENU));
				close.button_press_event.connect (on_button_press_event);
				box.pack_start (close, false, false, 0);

				close.margin_left = 3;
				
				this.add (box);
			}

			private bool on_button_press_event(Gdk.EventButton event) {
				tagparent.tags.remove (this);
				tagparent.tag_box.remove (this);
				tagparent.tag_removed(this.tag);
				return false;
			}
		}
		
		public ArrayList<Tag> tags { get; private set; }
		public Gtk.Entry add_tag_entry { get; private set; }

		private FootnoteApp app;
		private Note noteparent;
		private HBox tag_box;

		public TaggingArea (Note parent, FootnoteApp app){
			this.app = app;
			this.noteparent = parent;
			this.height_request = 30;

			tags = new ArrayList<Tag> ();

			tag_box = new HBox (false, 0);
			
			add_tag_entry = new Gtk.Entry ();
			add_tag_entry.placeholder_text = ("Add tag...");
			add_tag_entry.width_chars = 10;
			
			add_tag_entry.key_press_event.connect ( (event)=> {
				if  (event.keyval == 65293 && add_tag_entry.text != ""){
					add_tag (add_tag_entry.text);
					add_tag_entry.text = "";
				}
				return false;
			});

			add_tag_entry.focus_in_event.connect ( (event) => {
				app.selected.note = noteparent;
				add_tag_entry.show ();
				return false;
			});
			
			add_tag_entry.focus_out_event.connect ( (event) => {
				add_tag_entry.hide ();
				return false;
			});

			add_tag_entry.show.connect ( () => {
				if (app.selected.note != noteparent) 
					add_tag_entry.hide();
			});
			
			this.pack_start (add_tag_entry, false, false, 0);
			this.pack_start (tag_box, false, false, 0);
		}
		
		/**
		 * Adds a tag
		 * @param tag The name of the tag
		 */
		public void add_tag (string tag){
			if (has_tag (tag))
				return;

			tag_added(tag);
			var t = new Tag (tag, this);
			this.tag_box.pack_start (t, false, false, 5);
			this.tag_box.reorder_child (t, 0);
			this.tag_box.show_all ();
			this.tags.insert (0, t);
		}

		/**
		 * Check whether this taggingarea contains a certain tag
		 * @param name the name of the tag
		 */
		public bool has_tag (string name) {
			foreach (Tag tag in tags)
				if (tag.tag == name)
					return true;

			return false;
		}
		
		/**
		 * signal indicating that a new tag was added
		 * @param name the name of the tag
		 */
		public signal void tag_added(string name);
		
		/**
		 * signal indicating that a tag was deleted
		 * @param name the name of the tag
		 */
		public signal void tag_removed(string name);
	}
}
	
